Imports System.ComponentModel

Public Class FormPreferences : Inherits Component : Implements IExtenderProvider

    Private Shared _GenerateTabSize As Integer = 3
    Private _ToolStrips As New HashSet(Of ToolStrip)
    Private _NewControls As New HashSet(Of Control)

    ''' <summary>
    ''' Doppelklick im Designer auf diese Property lst Code-Generierung fr ToolstripItems aus
    ''' </summary>
    <Browsable(True)> _
    <Category("Verhalten")> _
    <DefaultValue(False)> _
    Public Property GenerateCode() As Boolean
        Get
            Return False
        End Get
        Set(ByVal value As Boolean)
            If value Then ExecuteGenerateCode(_ToolStrips)
        End Set
    End Property

    <Browsable(True)> _
    <Category("Verhalten")> _
    <DefaultValue(3)> _
    Public Property GenerateTabSize() As Integer
        Get
            Return _GenerateTabSize
        End Get
        Set(ByVal value As Integer)
            _GenerateTabSize = value
        End Set
    End Property

    Public Function CanExtend(ByVal extendee As Object) As Boolean Implements IExtenderProvider.CanExtend

        'Toolstrips merken, fr evtl. CodeGenerierung
        If TypeOf extendee Is ToolStrip Then _ToolStrips.Add(DirectCast(extendee, ToolStrip))

        'Diese OrElse-Kette arbeitet prinzipiell gleich wie eine ElseIf-Kette: sobald eine der
        ' definierten Bedingungen matcht wird die Kette verlassen.
        'Die Bedingung besteht darin, da TryAction prft, ob extendee vom angegebenen Typ ist,
        ' und der Delegat ist die dann einmalig auszufhrende Preferenz.
        'z.B. bei Labels willich .AutoSize=False haben, und von .HideSelection halte ich bei
        ' keinem Control etwas, und der voreingestellte Split-Balken des SplitContainers ist mir
        ' mit 4Pix viel zu schmal.
        Dim Dum = _
        TryAction(extendee, Function(Obj As SplitContainer) Obj.SplitterWidth.Assign(8)) OrElse _
        TryAction(extendee, Function(Obj As Label) Obj.AutoSize.Assign(False)) OrElse _
        TryAction(extendee, Function(Obj As TreeView) Obj.HideSelection.Assign(False)) OrElse _
        TryAction(extendee, Function(Obj As TextBoxBase) Obj.HideSelection.Assign(False)) OrElse _
        TryAction(extendee, Function(Obj As ListView) Obj.HideSelection.Assign(False))
    End Function

    ''' <summary> Falls Obj vom Typ T ist wird die Action(Of T) damit ausgefhrt </summary>
    ''' <returns> True, falls Obj vom Typ T ist </returns>
    Private Function TryAction(Of T As Control)( _
          ByVal Obj As Object, ByVal Preference As Action(Of T)) As Boolean
        'Nur fr **neue** Controls drfen die Preferenzen ausgefhrt werden.
        'neue Controls werden durch Test auf .Parent=Nothing identifiziert.
        'leider ist fr ToolstripItem keine Preferenz definierbar, da nicht von Control erbend.
        'ToolstripItem hat auch keine Property mit der man neue Item identifizieren kann.
        TryAction = TypeOf Obj Is T
        If TryAction Then
            Dim Ctl = DirectCast(Obj, T)
            If Ctl.Parent Is Nothing Then
                _NewControls.Add(Ctl)
            Else
                If _NewControls.Remove(Ctl) Then Preference(Ctl)
            End If
        End If
    End Function

    Public Shared Sub ExecuteGenerateCode(ByVal _ToolStrips As HashSet(Of ToolStrip))
        Dim AllWriter As New List(Of String)
        Dim Clickables As New List(Of String)
        For Each CustomerStrip In _ToolStrips
            Clickables.Clear()
            For Each Itm As ToolStripItem In CustomerStrip.Items
                CollectClickables(Itm, Clickables)
            Next
            If Clickables.Count > 0 Then
                Dim sHandles = Clickables.ToStringX(".Click, ") & ".Click"
                Dim sCase = Tab & Tab & "case Sender Is "
                Dim sCases = sCase & Clickables.ToStringX(NewLine & NewLine & sCase) & NewLine
                With AllWriter
                    .AddX("Private Sub ", CustomerStrip.Name, "_MenuClicked(ByVal Sender As Object, ByVal e As EventArgs) _")
                    .AddX(Tab, Tab, "Handles ", sHandles)
                    .AddX(Tab, "select case True")
                    .AddX(sCases)
                    .AddX(Tab, "End select")
                    .AddX("End Sub")
                    .AddX("")
                End With
            End If
        Next
        'Die Ausgabe wird in eine TextDatei geschrieben, die dann geffnet wird
        Dim S = AllWriter.ToStringX(NewLine).Replace(Tab, New String(" "c, _GenerateTabSize))
        Dim FileFullName As String = Environment.CurrentDirectory & "\LaunchEditorWith.txt"
        My.Computer.FileSystem.WriteAllText(FileFullName, S, False)
        System.Diagnostics.Process.Start(FileFullName).Dispose()
    End Sub

    Private Shared Sub CollectClickables(ByVal Itm As ToolStripItem, ByVal Clickables As List(Of String))
        If TypeOf Itm Is ToolStripDropDownItem Then
            With DirectCast(Itm, ToolStripDropDownItem)
                If .HasDropDownItems Then
                    For Each itmRecurse As ToolStripItem In .DropDownItems
                        CollectClickables(itmRecurse, Clickables)        ' Rekursion!
                    Next
                    Return
                End If
            End With
            Clickables.Add(Itm.Name)
        ElseIf TypeOf Itm Is ToolStripButton Then
            Clickables.Add(Itm.Name)
        End If
    End Sub

End Class
